library(glmnet)

data = read.csv("tecator.csv")

n = dim(data)[1]
set.seed(12345)
id = sample(1:n, floor(n * 0.5))
train = data[id, ]
test = data[-id, ]

# ----1.----

y_train <- train[, 102]
X_train <- train[, 2:101]
y_test <- test[, 102]
X_test <- test[, 2:101]

channels <- colnames(X_train)

formula <- as.formula(paste("Fat~", paste(channels, collapse = " + ")))

model <- lm(formula, data = train)

mse <- function(y, y_hat)
  mean((y_hat - y) ^ 2)

train_pred <- predict(model, train)
test_pred <- predict(model, test)

print(paste("MSE on the training data:", mse(train$Fat, train_pred)))
print(paste("MSE on the test data:", mse(test$Fat, test_pred)))


#----3.----


lambda_with_k_features = function(x_train, y_train, alpha, k = 3) {
  fit_train <- glmnet(as.matrix(X_train), y_train, alpha = alpha)
  
  plot(fit_train, xvar = "lambda", label = TRUE)
  grid()
  
  coef_matrix <- as.matrix(coef(fit_train))[-1, ] # ignore intercept
  
  lambda_values <- fit_train$lambda
  
  # sum of num of cols wit non zero coeff, i.e. number of features
  num_non_zero <- apply(coef_matrix != 0, 2, sum)
  
  # lambda value with k non zero coeff, i.e. number of features
  lambda_values[num_non_zero == k]
  
}

lambda_with_k_features(x_train, y_train, alpha = 1, k = 3)


#----4.----

lambda_with_k_features(x_train, y_train, alpha = 0, k = 3)

#----5.----

fit <- cv.glmnet(as.matrix(X_train), y_train, alpha = 1)

plot(fit, xvar = "lambda", label = TRUE)
grid()

fit$lambda.min

sum(coef(fit, s = "lambda.min")[-1] != 0) # ignore intercept

y_hat = predict(fit, newx = as.matrix(X_test), s = "lambda.min")

plot(
  y_test,
  y_hat,
  xlab = "Observed test values",
  ylab = "Predicted test values",
  main = "Original vs. predicted test values ",
  col = "blue"
)
grid()